#ifndef JOS_INC_TRAP_H
#define JOS_INC_TRAP_H

// these are processor defined
#define T_DIVIDE     0		// divide error
#define T_DEBUG      1		// debug exception
#define T_NMI        2		// non-maskable interrupt
#define T_BRKPT      3		// breakpoint
#define T_OFLOW      4		// overflow
#define T_BOUND      5		// bounds check
#define T_ILLOP      6		// illegal opcode
#define T_DEVICE     7		// device not available 
#define T_DBLFLT     8		// double fault
/* #define T_COPROC  9 */	// reserved (not generated by recent processors)
#define T_TSS       10		// invalid task switch segment
#define T_SEGNP     11		// segment not present
#define T_STACK     12		// stack exception
#define T_GPFLT     13		// genernal protection fault
#define T_PGFLT     14		// page fault
/* #define T_RES    15 */	// reserved
#define T_FPERR     16		// floating point error
#define T_ALIGN     17		// aligment check
#define T_MCHK      18		// machine check
#define T_SIMDERR   19		// SIMD floating point error

/* These are arbitrarily chosen, but with care not to overlap
 * processor defined exceptions or interrupt vectors.
 */
#define T_SYSCALL   48		// system call
#define T_DEFAULT   500		// catchall

#ifndef __ASSEMBLER__

#include <inc/types.h>

struct Trapframe {
	register_t tf_edi;
	register_t tf_esi;
	register_t tf_ebp;
	register_t tf_oesp;		/* Useless */
	register_t tf_ebx;
	register_t tf_edx;
	register_t tf_ecx;
	register_t tf_eax;
	segment_t tf_es;
	uint16_t tf_padding1;
	segment_t tf_ds;
	uint16_t tf_padding2;
	uint32_t tf_trapno;
	/* below here defined by x86 hardware */
	uint32_t tf_err;
	uintptr_t tf_eip;
	segment_t tf_cs;
	uint16_t tf_padding3;
	register_t tf_eflags;
	/* below only when crossing rings(e.g. user to kernel) */
	uintptr_t tf_esp;
	segment_t tf_ss;
	uint16_t tf_padding4;
};

#endif /* !__ASSEMBLER__ */

#endif /* !JOS_INC_TRAP_H */
